unit TapiDial;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Tapi,WinTypes, WinProcs,StdCtrls,ExtCtrls;
const
  HiVer = $00020000;  // Highest API version wanted (2.0)
  LoVer = $00010004;
type
  TCallDialog=(None,Win95);
  TStatusWindow=Boolean;
  TLanguage = (english, croatian);
  TTapiDial = class(TComponent)
  private
    { Private declarations }
    frmStatWindow : TForm;
    lbxStatWindow : TListBox;
    FTelephonNo : string;
    FCallDialog:TCallDialog;
    FStatusWindow:TStatusWindow;
    FLanguage : TLanguage;
    FOwnerFormHandle : hWnd;
    lineApp: THLineApp;
    line: THLine;
    call: THCall;
    //CallParams:TlineCallParams;
    CallParams:TLineCallParams;
    procedure TerminateTapi;
    procedure InitializeTapi;
    procedure SetLanguage(Value:TLanguage);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Dial;
    procedure HangUp;
    procedure SetCallDialog(NewCallDialog:TCallDialog);
    procedure SetTelephonNo(Value :string);
    procedure SetStatusWindow(NewStatusWindow:TStatusWindow);
    procedure TranslateDialog;
    procedure ConfigDialog;
  published
    { Published declarations }
    property TelephonNo: String
       read FTelephonNo
       write SetTelephonNo;
    property CallDialog:TCallDialog
       read FCallDialog
       write SetCallDialog;
    property StatusWindow:TStatusWindow
       read FStatusWindow
       write SetStatusWindow;
    property Language:TLanguage
       read FLanguage
       write SetLanguage;
  end;

  procedure CreateStatusWindow;
  procedure WriteStatus(Text:string);
  procedure DestroyStatusWindow;
 var
    buf:array[0..1023] of char;
    //CallInfo se puni u LineCallBack prilikom TapiInitialize
    callinfo: TLineCallInfo absolute buf;
    LineSt: string;
    StatusWindowCreated:Boolean;
    frmStatus:TForm;
    lbxStatus:TListBox;
    Lang : TLanguage;
    tapiVersion: Longint;
    varTelefon :string;
procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TTapiDial]);
end;

procedure TTapiDial.TranslateDialog;  // Tone/Pulse Dialing
Var
   TVersion: DWORD;
begin

  if tapiVersion <= LoVer  then
     TVersion := LoVer
  else if tapiVersion > LoVer then
     TVersion := HiVer;

    lineTranslateDialog(LineApp, 0, TVersion, FOwnerFormHandle, PChar (FTelephonNo));
end;

procedure TTapiDial.ConfigDialog;   //DialTone
begin
  lineConfigDialog(0, FOwnerFormHandle, 'comm/datamodem');
end;

//Kreira se nevidljiva forma
procedure CreateStatusWindow;
begin
//Kreiranje forme za StatusDialog
   if not StatusWindowCreated then begin
      frmStatus := TForm.Create(Application);
      StatusWindowCreated:=True;
      with frmStatus do begin
          Visible := False;
          Caption := 'Dial Stauts';
          if Lang= Croatian then Caption:='Status Linije';
          FormStyle := fsStayOnTop;
          BorderStyle := bsSizeable;
          BorderIcons := [];
          Width := 230;
          Height:=100;
          Left:=527;
          Top:=419;
          VertScrollBar.Visible:=True;
      end;
      lbxStatus := TListBox.Create(frmStatus);
      with lbxStatus do begin
        begin
          Parent := frmStatus;
          Align := alClient;
          Sorted := False;
          Font.Name := 'MS Sans Serif';
          Font.Size := 8;
        end;
      end;
   end//if not StatusWindowCreated
else StatusWindowCreated:=False;
//Kraj od kreiranje forme za StatusDialog
end;

procedure DestroyStatusWindow;
begin
   frmStatus.Release;
   StatusWindowCreated:=False;
   //FStatusWindow:=False;
end;

procedure TTapiDial.SetTelephonNo(Value :string);
begin
   if FTelephonNo <> Value then
      FTelephonNo:=Value;
   varTelefon:=FTelephonNo;
end;

procedure WriteStatus(Text:string);
begin
   Text:='-> '+Text;
   //Da ne petljamo po memoriji koja nije alocirana
   if not StatusWindowCreated then
      Exit
   else if StatusWindowCreated then  begin
      lbxStatus.Items.Add(Text);
      lbxStatus.ItemIndex := lbxStatus.Items.Count-1;
   end;
end;

procedure lineCallback(hDevice, dwMsg, dwCallbackInstance,
		dwParam1, dwParam2, dwParam3: LongInt);
		stdcall;
var
   s: string;
   hCall: THCall;
begin
//if not StatusWindowCreated then CreateStatusWindow;
if dwMsg = LINE_REPLY then { result of LineMakeCall }
   if dwParam2 < 0 then begin
      LineSt:='Reply error';
      if Lang = Croatian then LineSt:='Linija nije odgovorila';
      WriteStatus(LineSt);
   end
   else begin
      LineSt:='LINE_REPLY ok';
      WriteStatus(LineSt);
   end
   else if dwMsg = LINE_CALLSTATE then begin
   { change in line state }
      hCall := THCall(hDevice);
      case dwParam1 of
         LINECALLSTATE_IDLE:{ call terminated }
            if hcall <> 0 then begin
               lineDeallocateCall(hCall);
               { you must deallocate the call }
               LineSt:='Idle - call deallocated';
               if Lang = Croatian then LineSt:='Poziv je odbaen';
               WriteStatus(LineSt);
            end;
         LINECALLSTATE_CONNECTED:{ Service connected }
            if hCall <> 0 then begin
               s := 'Connected: '+ varTelefon + ' ';
               if Lang = Croatian then s:='Veza je uspostavljena: '+varTelefon+ ' ';
               callinfo.dwTotalSize := 1024;
               if lineGetCallInfo(hCall, callinfo) = 0 then
                  if callinfo.dwAppNameSize > 0 then
                     s := s + (buf + callinfo.dwAppNameOffset);
               LineSt:=s;
               WriteStatus(LineSt);
            end;
         LINECALLSTATE_PROCEEDING:
         begin{ call proceeding (dialing) }
            LineSt:='Proceeding';
            if Lang = Croatian then LineSt:='Nastavljam';
            WriteStatus(LineSt);
         end;
         LINECALLSTATE_DIALING:
         begin	{ dialing }
            LineSt:='Dialing: '+ varTelefon;
            if Lang = Croatian then LineSt:='Zovem: ' + varTelefon;
            WriteStatus(LineSt);
         end;
         LINECALLSTATE_DISCONNECTED:
         begin { disconnected }
            s := 'Disconnected: ';
            if Lang = Croatian then s:='Linija je iskljuena: ';
            if dwParam2 = LINEDISCONNECTMODE_NORMAL then
               s := s + 'normal';
               if Lang = Croatian then s:=s + 'normalno'
            else if dwParam2 = LINEDISCONNECTMODE_BUSY then
               s := s + 'busy';
               if Lang = Croatian then s:=s + ' zbog zauzea';
            LineSt:=s;
            WriteStatus(LineSt);
         end;
         LINECALLSTATE_BUSY:
         begin { busy }
            LineSt:='Busy';
            if Lang = Croatian then LineSt:='Linija je zauzeta';
            WriteStatus(LineSt);
         end;
      end;
   end;
end;//Kraj od lineCallBack

procedure TTapiDial.InitializeTapi;
var
   nDevs: Longint;
   extid: TLineExtensionID;
   Greska:Boolean;
begin
   Greska:=False;
   LineSt:='Initializing TAPI';
   if FLanguage = Croatian then LineSt:='Inicijaliziram TAPI';
   //Ako je postavljen zahtjev za StatusWindow i on je kreiran
   {if FStatusWindow and StatusWindowCreated then
      WriteStatus(LineSt)
   else begin
      StatusWindowCreated:=False;
   end; }
   WriteStatus(LineSt);
   FillChar(CallParams, sizeof(CallParams), 0);
   with CallParams do begin
      dwTotalSize := sizeof(CallParams);
      dwBearerMode := LINEBEARERMODE_VOICE;
      if FCallDialog = Win95 then //Upotrijebi dialog
         dwMediaMode := LINEMEDIAMODE_INTERACTIVEVOICE
      else if FCallDialog = None then begin//Bez dialoga
         dwMediaMode := LINEMEDIAMODE_DATAMODEM;
      end;
   end;
   if lineInitialize(lineApp, HInstance,@lineCallback,nil, nDevs)<0 then begin
      lineApp := 0;
      Greska:=True;
   end
   else if nDevs = 0 then begin
      lineShutDown(lineApp);    //No TAPI devices
      lineApp := 0;
      Greska:=True;
      LineSt:='No TAPI devices';
      if FLanguage = Croatian then LineSt:='Nemate instaliranih TAPI ureaja';
      WriteStatus(LineSt);
   end
   else if lineNegotiateAPIVersion(lineApp, 0,LoVer, HiVer,
   tapiVersion, extid) < 0 then begin
      lineShutDown(lineApp);
      lineApp := 0;
      Greska:=True;
      LineSt:='Error in lineNegotiateAPIVersion';
      if FLanguage = Croatian then LineSt:='Nemate potrebnu TAPI verziju';
      WriteStatus(LineSt);
   end
   else if lineOpen(lineApp, LINEMAPPER, line, tapiVersion, 0, 0,
           LINECALLPRIVILEGE_NONE, 0, @CallParams) < 0 then begin
      lineShutDown(lineApp);
      lineApp := 0;
      line := 0;
      Greska:=True;
      LineSt:='Error in LineOpen';
      if FLanguage = Croatian then LineSt:='Greka u otvaranju linije';
      WriteStatus(LineSt);
   end;
   if line = 0 then  begin
      LineSt:='Error!!';
      if FLanguage = Croatian then LineSt:='Greka, nema linija';
      Greska:=True;
      WriteStatus(LineSt);
   end;
   if not Greska then begin
      LineSt:='TAPI successfully initialized';
      if FLanguage = Croatian then LineSt:='TAPI je uspjeno inicijaliziran';
      WriteStatus(LineSt);
   end;
end;//InitializeTapi

constructor TTapiDial.Create(AOwner: TComponent);
var
   i:integer;
begin
   inherited Create(AOwner);
   //First check if Owner is TForm
   if not (AOwner is TForm) then
      Raise Exception.Create('Please drop the component onto a Form!');

  { then check, if this component is already on the form.}
   for i:=0 to TForm(AOwner).ComponentCount-1 do begin
      if (TForm(AOwner).Components[i].Classtype=TTapiDial)
         and (TForm(AOwner).Components[i]<>Self) then begin
         { if a component exists then don't of a new one }
            Raise Exception.Create('There is TTApiDial component already on this form');
         end;
   end;

   FOwnerFormHandle:=TForm(AOwner).Handle;
   FTelephonNo:='9865';
   FCallDialog:=None;
   CreateStatusWindow;
   FStatusWindow:=True;//To make the form Visible
   StatusWindowCreated:=True;
   InitializeTapi;
   FLanguage:=English;
end;//Kraj od Create

procedure TTapiDial.TerminateTapi;
begin
   { Terminate TAPI }
   if line <> 0 then
      lineClose(line);
   if lineApp <> 0 then
      lineShutDown(lineApp);
   LineSt:='TAPI terminated';
   if FLanguage = Croatian then LineSt:='Zavrili smo sa TAPI-jem.';
   WriteStatus(LineSt);
end;

destructor TTapiDial.Destroy;
begin
   FStatusWindow:=False;
   StatusWindowCreated:=False;
   TerminateTapi;
   inherited Destroy;
end;

procedure TTapiDial.Dial;
var
   c: array[0..30] of char;
   B:integer;
   St:string;
begin
   if Length(FTelephonNo) > 0 then begin
      StrPCopy(c, FTelephonNo);
      if lineMakeCall(line, call, c, 0, @CallParams) < 0 then  begin
         LineSt:='Error in lineMakeCall';
         if FLanguage = Croatian then LineSt:='Greka u zvanju';
         WriteStatus(LineSt);
      end;
      //Treba pokazati DialogBox, za HangUp, ako zovemo sa LINEMEDIAMODE_DATAMODEM
      if CallDialog=None then begin
         //Neemo ga pokazati ako je dolo do greke
         if (LineSt<>'Error in lineMakeCall') or (LineST<>'Greka u zvanju') then begin
            St:='HangUp?';
            if FLanguage = Croatian then
               St:='Kada zavrite razgovor, pritisnite dugme da prekinete vezu.';
            B:= Application.MessageBox(PChar(St)
                        ,PChar('Talking to ' + FTelephonNo), mb_OK);
            if B= IDOK then HangUp;
         end;
      end;//if CallDialog=None
      //Ako imamo greku za bilo koji CallDialog
      if (LineSt='Error in lineMakeCall') or (LineST='Greka u zvanju') then begin
         St:='Error during dialing?!';
         if FLanguage = Croatian then St:='Greka u zvanju';
         if MessageDlg(St,mtCustom,
                       [mbOK],0)=mrOK then Exit;

      end;

   end;//if Length(FTelephonNo) > 0
end; //Kraj od Dial


procedure TTapiDial.HangUp;
begin
   if LineDrop(call, nil, 0) < 0 then begin
      LineSt:='Error in lineDrop';
      if FLanguage = Croatian then LineSt:='Greka u zatvaranju linije.';
      WriteStatus(LineSt);
   end
   else begin
      LineSt:='Line Dropped';
      if FLanguage = Croatian then LineSt:='Linija je zatvorena.';
      WriteStatus(LineSt);
   end;
end;

procedure TTapiDial.SetCallDialog(NewCallDialog:TCallDialog);
var
   x:string;
begin
   if FCallDialog<>NewCallDialog then begin
      FCallDialog := NewCallDialog;
      case NewCallDialog of
         Win95: x:='Win95';
         None:  x:='None';
      end;
      LineSt:='Call dialog set to: ' + x;
      if FLanguage = Croatian then LineSt:='Dijalog za poziv je promjenjen u: ' + x;
      WriteStatus(LineSt);
      TerminateTapi;
      InitializeTapi;
   end;
end;

procedure TTapiDial.SetLanguage(Value:TLanguage);
begin
   if FLanguage <> Value then
      FLanguage:=Value;
   Lang:=FLanguage;
end;

procedure TTapiDial.SetStatusWindow(NewStatusWindow:TStatusWindow);
begin
  if FStatusWindow<>NewStatusWindow   then begin
     FStatusWindow := NewStatusWindow;
     //Bio je True, a sada je False
     if FStatusWindow=False then begin
        //Treba formu staviti Visible:=False
        //frmStatus.Visible:=False;
        //StatusWindowCreated:=False;
        if StatusWindowCreated then DestroyStatusWindow;
     end
     else if FStatusWindow = True then begin
        //frmStatus.Visible:=True;
        //StatusWindowCreated:=True;
        if not StatusWindowCreated then CreateStatusWindow;
        frmStatus.Visible:=True;
     end;
   end;
end;
initialization
   tapiVersion:=0;
   varTelefon:='';
end.
